(*
  Programme de DooMeeR - TodoList, "A Faire"
  Ce programme permet tout simplement de maintenir une liste de choses  faire,
  de la manire la plus simple et la plus rapide possible.
  - entrer un texte dans le champ Edit puis cliquer sur Ajouter pour ajouter
  un "todo" (un truc  faire quoi)
  - clic-gauche sur un "todo" pour le cocher / dcocher (coch = "fait")
  - clic-droit pour supprimer le "todo"
*)
unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls;

const ListeFichier = 'todos.txt'; // Fichier de sauvegarde
      TitreFenetre = 'A faire';   // Caption de la fentre

type
  TMain = class(TForm)
    TextToAdd: TEdit;
    Add: TButton;
    Liste: TListView;
    procedure AddClick(Sender: TObject);
    procedure TextToAddKeyPress(Sender: TObject; var Key: Char);
    procedure FormResize(Sender: TObject);
    procedure ListeMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure ListeKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure AddKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Dclarations prives }
  public
    procedure LoadListe(Fic: string);
    procedure SaveListe(Fic: string);
    procedure MajStatut;
  end;

var
  Main: TMain;

implementation

{$R *.DFM}

procedure TMain.AddClick(Sender: TObject);
begin
  // Ajoute un item  la liste
  with Liste.Items.Add do
  begin
    // Texte de la chose  faire
    Caption:=TextToAdd.Text;
    // On vide l'Edit (pratique)
    TextToAdd.Text:='';
    // On dmarre dcoch
    SubItems.Add('');
  end;
  MajStatut;
end;

procedure TMain.TextToAddKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#13 then // Touche "ENTREE"
  begin
    AddClick(Self);
    Key:=#0; // Pour viter le "BEEP"
  end;
  if Key=#27 then Close; // Touche "ECHAP"
end;

procedure TMain.FormResize(Sender: TObject);
begin
  // On redimensionne automatiquement la colonne "Texte" pour qu'elle prenne
  // le maximum de place disponible
  Liste.Columns[0].Width:=Liste.Width-Liste.Columns[1].Width-4;
end;

procedure TMain.ListeMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if ssLeft in Shift then // Bouton gauche
  begin
    // Cocher ou dcocher l'item slectionn
    if Liste.Selected<>nil then // Si on a un item slectionn (ici cliqu)...
      if Liste.Selected.SubItems[0]='X' then // S'il est coch (colonne "Fait" contient "X")...
        Liste.Selected.SubItems[0]:='' else Liste.Selected.SubItems[0]:='X'; // On vide, sinon, on met le X
    // On dslectionne (plus joli)
    Liste.Selected:=nil;
  end else
  if ssRight in Shift then // Bouton droit
  begin
    // Suppression de l'item slectionn s'il y en a un de slectionn
    if Liste.Selected<>nil then Liste.Selected.Free;
  end;
  // L'Edit reprends le Focus (il le perd donc le moins possible, c'est plus pratique)
  TextToAdd.SetFocus;
  MajStatut;
end;

procedure TMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // Sauvegarde automatique  la fermeture
  SaveListe(ListeFichier);
end;

procedure TMain.FormCreate(Sender: TObject);
begin
  // Chargement automatique  l'ouverture si le fichier de sauvegarde existe
  if FileExists(ListeFichier) then LoadListe(ListeFichier) else
  begin
    // Si le fichier n'existe pas, c'est srement le 1er chargement de l'appli
    // Donc on affiche une petite aide
    ShowMessage('Bienvenue dans ce petit programme!'#13#13+
                'Il vous permettra de maintenir une liste de trucs  faire, des "TODOS", le plus simplement possible.'#13+
                '- entrez un texte dans le champ texte puis cliquez sur Ajouter ou faites ENTREE pour ajouter un TODO'#13+
                '- faites un clic-gauche sur un TODO pour le cocher ou le dcocher ("fait" ou "pas fait")'#13+
                '- faites un clic-droit sur un TODO pour le supprimer');
  end;
  MajStatut;
end;

procedure TMain.LoadListe(Fic: string);
var F: TextFile;
    s: string;
begin
  assignfile(F, Fic);
  reset(F);
  // On vide la liste avant de la charger
  Liste.Items.Clear;
  while not eof(F) do // Tant qu'on arrive pas au bout du fichier...
  begin
    readln(F, s); // On lit une ligne dans s...
    if (length(s)>0) then // Si la ligne n'est pas vide...
    begin
      with Liste.Items.Add do // On ajoute un item dans la liste
      begin
        // La ligne doit commencer par X si coch, par un espace sinon (c'est S[1])
        // Le reste (c'est  dire la copie (copy) du caractre 2  la fin, contient le contenu
        // de la colonne "Texte"
        Caption:=copy(S, 2, length(S)-1); // Colonne "Texte" (1ere colonne = Caption)
        SubItems.Add(Trim(S[1])); // Colonne "Fait" (2eme colonne = SubItems[0])
      end;
    end;
  end;
  closefile(F);
  MajStatut;
end;

procedure TMain.SaveListe(Fic: string);
var F: TextFile;
    i: integer;
begin
  assignfile(F, Fic);
  rewrite(F);
  for i:=0 to Liste.Items.Count-1 do // Du 1er au dernier lment de la liste
  begin
    // On sauvegarde l'lment Liste.Items[i]
    // 1er caractre: fait ' ' ou pas fait 'X'
    // Suite de la ligne  crire: contenu de la colonne "Texte"
    if length(Liste.Items[i].SubItems[0]) > 0 then
      writeln(F, Liste.Items[i].SubItems[0][1]+Liste.Items[i].Caption)
    else
      writeln(F, ' '+Liste.Items[i].Caption)
  end;
  closefile(F);
end;

procedure TMain.MajStatut;
var i, c: integer;
begin
  // Compter le nombre de "todos" faits et mettre le rsultat dans c
  c:=0;
  for i:=0 to Liste.Items.Count-1 do // Pour chaque item...
    if Trim(Liste.Items[i].SubItems[0])<>'' then inc(c); // Si <>'', c'est  dire ='X', ajouter 1  c
  // On affiche le rsultat
  Caption:=TitreFenetre+' ('+inttostr(c)+'/'+inttostr(Liste.Items.Count)+')';
  // Pour que a s'affiche aussi dans la barre des tches
  Application.Title:=Caption;
end;

procedure TMain.ListeKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=27 then Close; // Valable aussi bien dans l'Edit que dans la liste et le bouton "Echap"
end;

procedure TMain.AddKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=27 then Close; // Valable aussi bien dans l'Edit que dans la liste et le bouton "Echap"
end;

end.
